import numpy as np
import pylab
from lmfit import Minimizer, Parameters, fit_report

#nacteni dat do matice
data=np.loadtxt('Data.dat')  
xdata=data[:,0]
ydata=data[:,1]

#definice residualu: rozdilu fitovane funkce a dat
def residual(params, x, ydata):
    v = params.valuesdict()    #rozbaleni parametru do slovniku v
    e = v['eINF']-v['w']**2/(x*(x+1j*v['g']))  #dielektricka funkce
    N = e**0.5      #index lomu
    R=abs((N-1)/(N+1))**2 #vypocet odrazivosti
    return R-ydata    

#vytvoreni parametru a jejich startovacich hodnot. Mozno nastavit, zda se parametr fituje (vary=True) nebo ne 
params = Parameters()
params.add('eINF', value=9,vary=True)
params.add('w', value=1780,vary=True)
params.add('g', value=300,vary=True)

#fitovani
minner = Minimizer(residual, params, fcn_args=(xdata,ydata))
result = minner.minimize(method='leastsq')

# calculate final result: data + residual
final = ydata + result.residual

# write error report, alternative result.params.pretty_print()
print(fit_report(result))

#ulozeni nafitovaneho spektra do souboru
FileData= open('Spektrum.dat', 'w')
for i in range(len(xdata)):
    print (xdata[i], "\t",final[i], file=FileData)

#ulozeni statistiky do souboru
FileFit = open('Statistika.dat', 'w')
print (fit_report(result), file=FileFit)


#pro dalsi praci s vysledky fitu se muze hodit:
FitParams=result.params  #slovnik s objekty Parameter s nafitovanymi hodnotami parametru a jejich chybami. 
print
print ("hodnota eINF: ",  FitParams['eINF'].value, " a jeho chyba",  FitParams['eINF'].stderr)
print ("hodnoty korelacnich koeficientu s dalsimi parametry: ", FitParams['eINF'].correl)

# vykresleni fitu na monitor
original= ydata+residual(params,xdata,ydata)
pylab.plot(xdata, original, 'g-', label='Original model')
pylab.plot(xdata, ydata, 'k-', label='data')
pylab.plot(xdata, final, 'r-', label='Fitted model')
legend = pylab.legend(loc='upper right', shadow=False, fontsize='x-small')
pylab.show()

